#include "tcpip.h"

struct realm *tcp_realm;

struct tcp_head *tcp_rules=NULL;
unsigned int num_tcp_head=0;
unsigned int num_tcp_rules=0;

void tcp_match(struct packet *p, unsigned int l)
{
	struct tcp_head *h;
	struct pkt_iphdr *iph;
	struct pkt_tcphdr *tcph;
	u_int16_t sp, dp;

	iph=p->layer[l].h.ip;
	tcph=p->layer[l+1].h.tcp;

	sp=ntohs(tcph->sport);
	dp=ntohs(tcph->dport);

	for(h=tcp_rules; h; h=h->next) {
		if ( 
			(((h->saddr&h->smask) == (iph->saddr&h->smask))
			^ (h->flags&NEGATE_SADDR) ) &&
			
			(((h->daddr&h->dmask) == (iph->daddr&h->dmask))
			^ (h->flags&NEGATE_DADDR) ) &&

			(((sp>=h->sport_min)&&(sp<=h->sport_max)))
			^ (h->flags&NEGATE_SPORT) &&
			
			(((dp>=h->dport_min)&&(dp<=h->dport_max)))
			^ (h->flags&NEGATE_DPORT)){

			tcpip_traverse(p, l, h->rules);
		}
	}
}

int tcp_commit(struct rule *r)
{
	return tcp_udp_commit(&tcp_rules,
		&num_tcp_head,
		&num_tcp_rules, r);
}

struct proto tcp_p=init_proto_ex("tcp", tcp_decode, tcp_dprint);
struct proto_req tcp_r[]={
	proto_request("ipv4", 6),
	null_request()
};

int tcp_dprint(struct layer *l, char *buf, int buflen)
{
	char tcpflags[]="FSRPAU12";
	char disp[9];
	int n, x;
	
	for(n=0,x=1; n<8; n++, x*=2) {
		if ( l->h.tcp->flags.flags & x ){
			disp[n]=tcpflags[n];
		}else{
			disp[n]='*';
		}
	}disp[8]='\0';
	
	return snprintf(buf, buflen,
		"SPT=%u DPT=%u [%s] SEQ=0x%x ACK=0x%x WIN=0x%x",
		htons(l->h.tcp->sport),
		htons(l->h.tcp->dport),
		disp,
		htonl(l->h.tcp->seq),
		htonl(l->h.tcp->ack),
		htons(l->h.tcp->win));
}

int tcp_decode(struct packet *p)
{
	struct proto_child *pc;
	struct layer *l=&p->layer[p->llen-1];
	int ret=0;

	if ( p->llen >= PKT_LAYERS ) return ret;

	/* FIXME: Check sizeof(struct pkt_tcphdr) */
	if ( (p->layer[p->llen].h.raw=
		l->h.raw+(l->h.tcp->doff<<2))
		>= p->end ) return ret;

	for(pc=l->proto->children; pc; pc=pc->next)
	{
		if ( l->h.tcp->dport == pc->id ||
			l->h.tcp->sport == pc->id ) {
			p->layer[p->llen++].proto=pc->proto;
			return pc->proto->decode(p) | ret;
		}
	}

	p->layer[p->llen++].proto=NULL;
	return ret;
}
