/*
 * This file is part of firesorm NIDS
 * Copyright (c) 2002 Gianni Tedesco
 *
 * 	This little stub is the signature engine. We construct rule objects
 * 	and pass them on to the relevent backends (realms). The real work
 * 	is then done inside the plugins. The functions here are intended
 * 	for parser plugins.
 */
#include <stdlib.h>
#include <string.h>
#include <cleanup.h>
#include <packet.h>
#include <target.h>
#include <signature.h>
#include <realm.h>

void signature_killtmp(struct critmp *c)
{
	struct critmp *foo, *bar;

	for(foo=c; foo;) {
		bar=foo;
		foo=foo->next;
		if ( bar->args) free(bar->args);
		free(bar->crit);
		free(bar);
	}
}

struct rule *signature_rule(char *rlm)
{
	struct rule *ret;

	if ( !(ret=calloc(1, sizeof(*ret))) ) {
		cperror("calloc");
	}

	if ( !(ret->realm=realm_find(rlm)) ) {
		free(ret);
		return NULL;
	}

	return ret;
}

void signature_abort(struct rule *r)
{
	if ( !r ) return;

	/* Target/alert stuff */
	if ( r->alert ) free(r->alert);
	signature_killtmp(r->criteria);
	if ( r->target && r->t_priv ) {
		r->target->cleanup(r->t_priv);
	}
	
	free(r);
}

int signature_criteria(struct rule *r, int n, char *c, char *val)
{
	struct critmp *crit;
	
	if ( !r ) return 0;
	if ( !c ) return 0;
	if ( !r->realm ) return 0;

	/* Copy all the data */
	if ( !(crit=calloc(1, sizeof(*crit))) )
		cperror("malloc");

	if ( !(crit->crit=strdup(c)) )
		cperror("strdup");

	if ( val && !(crit->args=strdup(val)) )
		cperror("strdup");

	crit->negate=n ? 1 : 0;
	
	/* Thwack it on the end of the list */
	crit->next=NULL;
	if ( !r->criteria ) {
		r->criteria=crit;
		r->crit_end=crit;
	}else{
		r->crit_end->next=crit;
		r->crit_end=crit;
	}

	return 1;
}

int signature_alert(struct rule *r, char *alert, char *text, char *priv)
{
	if ( !r ) return 0;

	/* Find the target */
	if ( !(r->target=target_find(alert)) )
		return 0;

	/* Validate the private data */
	if ( !r->target->validate(priv, &r->t_priv) )
		return 0;
	
	/* Set the text */
	if ( !(r->alert=strdup(text)) )
		cperror("strdup");

	return 1;
}

int signature_commit(struct rule *r)
{
	if ( !r ) return 0;
	if ( !r->realm ) return 0;
	if ( !r->realm->commit ) return 0;

	if ( r->realm->commit(r) ) {
		signature_killtmp(r->criteria);
		free(r);
		return 1;
	}else return 0;
}
